import os
from datetime import datetime

import matplotlib.pyplot as plt
import pandas as pd
from sklearn.feature_extraction.text import CountVectorizer
from wordcloud import WordCloud


class TextAnalysis:

    @staticmethod
    def generate_word_frequency(corpus, top_n=None):
        now = int(round(datetime.now().timestamp() * 1000))
        if not os.path.exists("output"):
            os.makedirs("output")

        vec = CountVectorizer().fit(corpus)
        bag_of_words = vec.transform(corpus)
        sum_words = bag_of_words.sum(axis=0)
        words_freq = [(word, sum_words[0, idx])
                      for word, idx in vec.vocabulary_.items()]
        words_freq = sorted(words_freq, key=lambda x: x[1], reverse=True)

        df = pd.DataFrame(words_freq, columns=["word", "count"])
        df.to_csv("output/wordfreq_%s.csv" % now, index=False)

        return words_freq[:top_n]

    @staticmethod
    def generate_word_cloud(corpus, show=False):
        now = int(round(datetime.now().timestamp() * 1000))
        if not os.path.exists("output"):
            os.makedirs("output")

        f = plt.figure(figsize=(40, 25))
        cloud = WordCloud(
            background_color='black',
            collocations=False,
            width=2500,
            height=1800
        ).generate(" ".join(corpus))
        plt.axis('off')
        plt.margins(x=0, y=0)
        plt.title("Word Cloud", fontsize=40)
        plt.imshow(cloud)
        f.savefig("output/wordcloud_%s.pdf" % now, bbox_inches='tight')

        if show:
            plt.show()
