import glob
import re
import string
from os import path

import contractions
import nltk
import pandas as pd
from nltk import PorterStemmer
from nltk.corpus import stopwords
from nltk.stem import WordNetLemmatizer
from nltk.tokenize import word_tokenize
import nltkmodules
from bs4 import BeautifulSoup


class TextNormalization:

    def __init__(self, custom_stopwords_file=None, programming_languages=None):
        nltkmodules.download_nltk_modules()
        self.programming_languages = programming_languages
        self.custom_stopwords_file = custom_stopwords_file
        self.stopwords = self.__get_stopword_list()

    def __get_stopword_list(self, ):
        # nltk stopwords
        stop_words = stopwords.words('english')

        # custom stopwords
        if self.custom_stopwords_file is not None:
            if path.exists(self.custom_stopwords_file):
                stopwords_custom = pd.read_csv(self.custom_stopwords_file)
                stop_words.extend(stopwords_custom['word'].values)

        # programming language specific stopwords (aka key/reserved words)
        if self.programming_languages is not None:
            programing_stopwords = glob.glob('stopwords/stopwords_programming_*.csv')
            for item in self.programming_languages:
                if item == 'all':
                    for file in programing_stopwords:
                        stopwords_programming = pd.read_csv(file)
                        stop_words.extend(stopwords_programming['word'].values)
                elif any(item in s for s in programing_stopwords):
                    stop_words_file = 'stopwords/stopwords_programming_' + item + ".csv"
                    if path.exists(stop_words_file):
                        stopwords_programming = pd.read_csv(stop_words_file)
                        stop_words.extend(stopwords_programming['word'].values)
                    else:
                        print("Stopwords file does not exist: ", stop_words_file)
                else:
                    print("Invalid programming language: ", item)

        stop_words_unique = set()
        [x for x in stop_words if x.lower() not in stop_words_unique and not stop_words_unique.add(x.lower())]

        return stop_words_unique

    def normalize_text(self, text, programming_language=None):
        # remove whitespaces from start and end of text
        text = text.strip()

        # expand contractions
        text = contractions.fix(text)
        
        # remove code blocks
        text = self.__remove_code_tags(text)

        # remove urls
        text = re.sub(r'\w+:/{2}[\d\w-]+(\.[\d\w-]+)*(?:(?:\/[^\s/]*))*', '', text)

        # remove html tags
        text = re.sub(r'<[^>]+>', '', text)

        # remove numbers
        text = re.sub(r'\d+', '', text)

        # remove special characters
        text = re.sub(r'/[.,\/#!$%\^&\*;:{}=\-_`~()]/g', '', text)
        text = re.sub(r'[!@#$%^&*(),.?":{}|<>-]', '', text)
        text = re.sub(r'[^a-zA-Z\d\s:]', '', text)

        # tokenize word
        tokens_text = word_tokenize(text)

        # remove single characters
        tokens_text = [i for i in tokens_text if not 1 == len(i)]

        # convert to lowercase
        tokens_text = [str.lower(x) for x in tokens_text]
        # text = text.lower()

        # remove stopwords
        tokens_text = [i for i in tokens_text if not i in self.stopwords]

        # remove punctuations
        tokens_text = [i for i in tokens_text if not i in string.punctuation]
        

        return tokens_text
        
    # Function to remove tags
    @staticmethod
    def __remove_code_tags(html): 
        # parse html content
        soup = BeautifulSoup(html, "html.parser")
  
        for data in soup(['code']):
            # Remove tags
            data.decompose()
  
    # return data by retrieving the tag content
        return ' '.join(soup.stripped_strings)


    @staticmethod
    def __regenerate_sentence(tokens):
        text = ""
        for i in tokens:
            text += i
            text += " "

        return text.strip()
